@@ -1,45 +1,39 @@
 use proc_macro2::TokenStream;
 
-use syn::{self, Data, Fields, Ident, Variant};
+use syn::{self, spanned::Spanned, Fields, Variant};
 
-use super::Context;
+use super::context::Context;
 
 pub fn transcoder_decorator(ast: &syn::DeriveInput) -> TokenStream {
     let ctx = Context::from_ast(ast);
-    let variants = match ast.data {
-        Data::Enum(ref data_enum) => &data_enum.variants,
-        Data::Struct(_) => panic!("NifUntaggedEnum can only be used with enums"),
-        Data::Union(_) => panic!("NifUntaggedEnum can only be used with enums"),
-    };
 
-    let num_lifetimes = ast.generics.lifetimes().count();
-    if num_lifetimes > 1 {
-        panic!("Enum can only have one lifetime argument");
-    }
-    let has_lifetime = num_lifetimes == 1;
+    let variants = ctx
+        .variants
+        .as_ref()
+        .expect("NifUntaggedEnum can only be used with enums");
 
     for variant in variants {
         if let Fields::Unnamed(_) = variant.fields {
             if variant.fields.iter().count() != 1 {
-                panic!("NifUntaggedEnum can only be used with enums that contain all NewType variants.");
+                return quote_spanned! { variant.span() =>
+                    compile_error!("NifUntaggedEnum can only be used with enums that contain all NewType variants.");
+                };
             }
         } else {
-            panic!(
-                "NifUntaggedEnum can only be used with enums that contain all NewType variants."
-            );
+            return quote_spanned! { variant.span() =>
+                compile_error!("NifUntaggedEnum can only be used with enums that contain all NewType variants.");
+            };
         }
     }
 
-    let variants: Vec<&Variant> = variants.iter().collect();
-
     let decoder = if ctx.decode() {
-        gen_decoder(&ast.ident, &variants, has_lifetime)
+        gen_decoder(&ctx, variants)
     } else {
         quote! {}
     };
 
     let encoder = if ctx.encode() {
-        gen_encoder(&ast.ident, &variants, has_lifetime)
+        gen_encoder(&ctx, variants)
     } else {
         quote! {}
     };
@@ -52,12 +46,9 @@ pub fn transcoder_decorator(ast: &syn::DeriveInput) -> TokenStream {
     gen
 }
 
-pub fn gen_decoder(enum_name: &Ident, variants: &[&Variant], has_lifetime: bool) -> TokenStream {
-    let enum_type = if has_lifetime {
-        quote! { #enum_name <'b> }
-    } else {
-        quote! { #enum_name }
-    };
+fn gen_decoder(ctx: &Context, variants: &[&Variant]) -> TokenStream {
+    let enum_type = &ctx.ident_with_lifetime;
+    let enum_name = ctx.ident;
 
     let variant_defs: Vec<_> = variants
         .iter()
@@ -66,7 +57,7 @@ pub fn gen_decoder(enum_name: &Ident, variants: &[&Variant], has_lifetime: bool)
             let field_type = &variant.fields.iter().next().unwrap().ty;
 
             quote! {
-                if let Ok(inner) = #field_type::decode(term) {
+                if let Ok(inner) = <#field_type>::decode(term) {
                     return Ok( #enum_name :: #variant_name ( inner ) )
                 }
             }
@@ -86,12 +77,9 @@ pub fn gen_decoder(enum_name: &Ident, variants: &[&Variant], has_lifetime: bool)
     gen
 }
 
-pub fn gen_encoder(enum_name: &Ident, variants: &[&Variant], has_lifetime: bool) -> TokenStream {
-    let enum_type = if has_lifetime {
-        quote! { #enum_name <'b> }
-    } else {
-        quote! { #enum_name }
-    };
+fn gen_encoder(ctx: &Context, variants: &[&Variant]) -> TokenStream {
+    let enum_type = &ctx.ident_with_lifetime;
+    let enum_name = ctx.ident;
 
     let variant_defs: Vec<_> = variants
         .iter()
